;Written by Iain McCurdy

<CsoundSynthesizer>

<CsOptions>
-odevaudio -b400
</CsOptions>

<CsInstruments>

sr 		= 	44100	;SAMPLE RATE
kr 		= 	441	;CONTROL RATE
ksmps 		= 	100	;NUMBER OF AUDIO SAMPLES IN EACH CONTROL CYCLE
nchnls 		= 	2	;NUMBER OF CHANNELS (2=STEREO)

;FLTK INTERFACE CODE;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;		LABEL      | WIDTH | HEIGHT | X | Y
	FLpanel	"pvsfread",   500,    150,    0,  0

;SWITCHES  	                         	ON | OFF | TYPE | WIDTH | HEIGHT | X | Y | OPCODE | INS | STARTTIM | DUR
gkOnOff,ihOnOff	FLbutton	"On/Off",	1,  -1,     2,    150,     30,     0,  0,    0,     1,      0,        -1

;VALUE DISPLAY BOXES		LABEL | WIDTH | HEIGHT | X | Y
idptr			FLvalue	" ",	 80,      20,    0,  80
idGain			FLvalue	" ",	 80,      20,    0, 130

;SLIDERS				            		MIN   | MAX | EXP | TYPE |  DISP   | WIDTH | HEIGHT | X  | Y
gkptr, ihptr		FLslider 	"File Pointer",		0,       1,    0,    3,    idptr,     500,    30,     0,   50
gkGain, ihGain		FLslider 	"Gain",			0,       1,    0,    3,   idGain,     500,    30,     0,  100

;SET INITIAL VALUES FOR SLIDERS 	|VALUE | HANDLE
			FLsetVal_i	 0.5,	ihGain
			FLsetVal_i	   0,	ihptr

			FLpanel_end

;INSTRUCTIONS AND INFO PANEL                 WIDTH | HEIGHT | X | Y
				FLpanel	" ", 500,     300,   512, 0
;TEXT BOXES												TYPE | FONT | SIZE | WIDTH | HEIGHT | X | Y
ih		 	FLbox  	"                          pvsfread                           ", 	1,      5,     14,    490,    15,     5,  0
ih		 	FLbox  	"-------------------------------------------------------------", 	1,      5,     14,    490,    15,     5,  20
ih		 	FLbox  	"Pvsfread is used to read data from a Pvoc-ex file            ", 	1,      5,     14,    490,    15,     5,  40
ih		 	FLbox  	"(extension .pvx) and convert it into an fsig which can       ", 	1,      5,     14,    490,    15,     5,  60
ih		 	FLbox  	"subsequently be transformed by many of Csound's fsig opcodes.", 	1,      5,     14,    490,    15,     5,  80
ih		 	FLbox  	"Pvsfread offers an input argument to define the location, in ", 	1,      5,     14,    490,    15,     5, 100
ih		 	FLbox  	"seconds, from the beginning of the file from which to read   ", 	1,      5,     14,    490,    15,     5, 120
ih		 	FLbox  	"data. In this example the slider range goes from 0 to 1 but  ", 	1,      5,     14,    490,    15,     5, 140
ih		 	FLbox  	"values are remapped within the instrument to cover the full  ", 	1,      5,     14,    490,    15,     5, 160
ih		 	FLbox  	"duration of the analysis file. This method allows easy       ", 	1,      5,     14,    490,    15,     5, 180
ih		 	FLbox  	"replacement of the given analysis file without the need to   ", 	1,      5,     14,    490,    15,     5, 200
ih		 	FLbox  	"modify file duration values. Examine the code for further    ", 	1,      5,     14,    490,    15,     5, 220
ih		 	FLbox  	"clarification.                                               ", 	1,      5,     14,    490,    15,     5, 240
ih		 	FLbox  	"The fsig output by pvsfread is resynthesised using the       ", 	1,      5,     14,    490,    15,     5, 260
ih		 	FLbox  	"pvsynth opcode.                                              ", 	1,      5,     14,    490,    15,     5, 280

				FLpanel_end

				FLrun	;RUN THE FLTK WIDGET THREAD
;END OF FLTK INTERFACE CODE;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		instr 		1
if		gkOnOff!=-1	goto	CONTINUE	;SENSE FLTK ON/OFF SWITCH            
		turnoff					;TURN THIS INSTRUMENT OFF IMMEDIATELY
CONTINUE:						;LABEL                               
#define		ANALYSISFILE	#"AndItsAll.pvx"#	;CREATE A TEXT MACRO OF THE CHOSEN ANALYSIS FILE NAME (TO SAVE US HAVING TO WRITE IT OUT TWICE!)
							;IF YOU WISH TO USE A DIFFERENT ANALYSIS FILE YOU ONLY NEED TO CHANGE THE REFERENCE HERE
ilen		filelen		$ANALYSISFILE			;DERIVE THE FILE LENGTH (IN SECONDS) OF THE CHOSEN ANALYSIS FILE	
;OUTPUT		OPCODE		POINTER     |   FILE
fsig  		pvsfread	gkptr * ilen, $ANALYSISFILE	;READ AN ANALYSIS FILE FROM THE HARD DRIVE ACCORDING TO THE GIVEN FILE POINTER LOCATION. OUTPUT AN F-SIGNAL.
;OUTPUT		OPCODE		INPUT
aresyn 		pvsynth  	fsig                      		;RESYNTHESIZE THE f-SIGNAL AS AN AUDIO SIGNAL
		outs		aresyn * gkGain, aresyn * gkGain	;SEND THE RESYNTHESIZED SIGNAL TO THE AUDIO OUTPUTS AND RESCALE USING ON-SCREEN GAIN CONTROL
		endin

</CsInstruments>

<CsScore>
f 0 3600	;DUMMY SCORE EVENT - ALLOWS REAL TIME PERFORMANCE FOR 1 HOUR
</CsScore>

</CsoundSynthesizer>