;Written by Iain McCurdy, 2008

<CsoundSynthesizer>

<CsOptions>
-odevaudio -b4096
</CsOptions>

<CsInstruments>

sr	=	44100
kr	=	44100
ksmps	=	1
nchnls	=	2

;FLTK INTERFACE CODE;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;			LABEL                                       | WIDTH | HEIGHT | X | Y
	FLpanel	"Phase Modulation Synthesis: Carrier(with feedback)",  500,    200,    0,  0

;                                                      ON | OFF | TYPE | WIDTH | HEIGHT | X | Y | OPCODE | INS | STARTTIM | IDUR
gkOnOff,ihOnOff	FLbutton	"On/Off",		1,   0,     2,    150,     28,    0,  0,    0,      1,      0,       3600
gkExit,ihExit	FLbutton	"Exit",			1,   0,     1,    150,     28,  350,  0,    0,    999,      0,       0.001

;VALUE DISPLAY BOXES					WIDTH | HEIGHT | X |  Y
idfreq				FLvalue	" ",     	60,       20,    0,   80
idAmp				FLvalue	" ",     	60,       20,    0,  130
idFBRatio			FLvalue	" ",     	60,       20,    0,  180

;					            			MIN  |   MAX | EXP | TYPE |   DISP     | WIDTH | HEIGHT | X | Y
gkfreq, ihfreq			FLslider 	"Base Frequency",	20,    20000,  -1,     3,   idfreq,        500,    30,     0, 50
gkAmp, ihAmp			FLslider 	"Amplitude",		0,     30000,   0,     3,   idAmp,         500,    30,     0, 100
gkFBRatio, ihFBRatio		FLslider 	"Feedback Ratio",	0,         1,   0,     3,   idFBRatio,     500,    30,     0, 150

			FLsetVal_i	100, 	ihfreq
			FLsetVal_i	20000, 	ihAmp
			FLsetVal_i	0.1, 	ihFBRatio

			FLpanel_end

;INSTRUCTIONS AND INFO PANEL
				FLpanel	" ", 500, 300, 512, 0
;TEXT BOXES												TYPE | FONT | SIZE | WIDTH | HEIGHT | X | Y
ih		 	FLbox  	"     Phase Modulation Synthesis: Carrier (with feedback)     ", 	1,      5,     14,    490,    15,     5,  0
ih		 	FLbox  	"-------------------------------------------------------------", 	1,      5,     14,    490,    15,     5,  20
ih		 	FLbox  	"This example implements phase modulation synthesis with a    ", 	1,      5,     14,    490,    15,     5,  40
ih		 	FLbox  	"single oscillator self-modulating via a feedback loop.       ", 	1,      5,     14,    490,    15,     5,  60
ih		 	FLbox  	"modulator.                                                   ", 	1,      5,     14,    490,    15,     5,  80
ih		 	FLbox  	"                            +------+                         ", 	1,      5,     14,    490,    15,     5, 100
ih		 	FLbox  	"                            V      |                         ", 	1,      5,     14,    490,    15,     5, 120
ih	  	 	FLbox  	"                         +--+--+   |                         ", 	1,      5,     14,    490,    15,     5, 140
ih		 	FLbox  	"                         |CAR.1|   |(feedback)               ", 	1,      5,     14,    490,    15,     5, 160
ih		 	FLbox  	"                         +--+--+   |                         ", 	1,      5,     14,    490,    15,     5, 180
ih		 	FLbox  	"                            |      |                         ", 	1,      5,     14,    490,    15,     5, 200
ih		 	FLbox  	"                            +------+                         ", 	1,      5,     14,    490,    15,     5, 220
ih		 	FLbox  	"                            |                                ", 	1,      5,     14,    490,    15,     5, 240
ih		 	FLbox  	"                            V                                ", 	1,      5,     14,    490,    15,     5, 260
ih		 	FLbox  	"                           OUT                               ", 	1,      5,     14,    490,    15,     5, 280

				FLpanel_end

				FLrun	;RUN THE FLTK WIDGET THREAD
;END OF FLTK INTERFACE CODE;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

gisine	ftgen	0,0,65537,10,1	;A SINE WAVE THAT CAN BE REFERENCED BY THE GLOBAL VARIABLE 'gisine'

instr 1
	if	gkOnOff!=0	kgoto	CONTINUE	;SENSE IF FLTK INSTRUMENT ON/OFF SWITCH IS NOT OFF (IN WHICH CASE SKIP THE NEXT LINE)
	turnoff						;TURN THIS INSTRUMENT OFF IMMEDIATELY
	CONTINUE:					;A LABEL

	iporttime	=	.02				;PORTAMENTO FUNCTION (WILL BE USED TO SMOOTH PARAMETER MOVED BY FLTK WIDGETS)
	kporttime	linseg	0,.001,iporttime,1,iporttime	;PORTAMENTO FUNCTION (WILL BE USED TO SMOOTH PARAMETER MOVED BY FLTK WIDGETS)
	
	kfreq		portk	gkfreq, kporttime		;APPLY SMOOTHING PORTAMENTO TO THE VARIABLE 'kbasefreq'
	kFBRatio	portk	gkFBRatio, kporttime		;APPLY SMOOTHING PORTAMENTO TO THE VARIABLE 'kFBratio'
	kAmp	portk	gkAmp, kporttime			;APPLY SMOOTHING PORTAMENTO TO THE VARIABLE 'gkAmp'

	aFB		init	0				;CREATE AN INITIAL VALUE FOR THE FEEDBACK SIGNAL
	aCarPhase	phasor	kfreq				;CREATE A MOVING PHASE VALUE THAT WILL BE USED TO READ CREATE THE CARRIER
	aCarrier	tablei	aCarPhase+aFB,gisine,1,0,1	;CARRIER OSCILLATOR IS CREATED. NOTE THAT FEEDBACK SIGNAL IS ADDED TO THE POINTER VARIABLE
	aFB		=	aCarrier * kFBRatio		;FEEDBACK SIGNAL UPDATED FOR THE NEXT PASS
	outs	aCarrier*kAmp, aCarrier*kAmp			;SEND THE AUDIO OUTPUT OF THE CARRIER WAVEFORM TO THE OUTPUTS - RESCALE WITH 'gkAmp' VARIABLE
endin

instr	999
	exitnow
endin

</CsInstruments>

<CsScore>
f 0 3600	;DUMMY SCORE EVENT - KEEPS REALTIME PERFORMANCE GOING FOR ONE HOUR
</CsScore>

</CsoundSynthesizer>