;Written by Iain McCurdy, 2006

<CsoundSynthesizer>

<CsOptions>
-idevaudio -odevaudio -m0 -d -M0 -b400
</CsOptions>

<CsInstruments>

sr 		= 	44100	;SAMPLE RATE
kr 		= 	4410	;CONTROL RATE
ksmps 		= 	10	;NUMBER OF AUDIO SAMPLES IN EACH CONTROL CYCLE (MAY NEED TO BE LOW WHEN WORKING WITH SHORT DELAY TIMES DEFINED INITIALLY AT KRATE)
nchnls 		= 	2	;NUMBER OF CHANNELS (2=STEREO)

;FLTK INTERFACE CODE;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;			LABEL                       | WIDTH | HEIGHT | X | Y
	FLpanel	"Keyboard Controlled Pitch Shifter",   500,    350,    0,  0

;VALUE_DISPLAY_BOXES		WIDTH | HEIGHT | X | Y
iddlt		FLvalue   " ",	88,       20,    0,  30
idfeedback	FLvalue   " ",	88,       20,    0,  80
iddry		FLvalue   " ",	88,       20,    0, 130
idwet		FLvalue   " ",	88,       20,    0, 180
idatt		FLvalue   " ",	88,       20,    0, 230
idrel		FLvalue   " ",	88,       20,    0, 280
idamp		FLvalue   " ",	88,       20,    0, 330

;SLIDERS					            					MIN | MAX | EXP | TYPE |    DISP     | WIDTH | HEIGHT | X | Y
gkdlt,ihdlt		FLslider	"Grain Size in Seconds (Used by Pitch Shifter)",  	.001, .2,    0,     5,        iddlt,    500,     26,    0,   0
gkfeedback,ihfeedback	FLslider	"Feedback",  						-.99, .99,   0,     5,   idfeedback,    500,     26,    0,  50
gkdry,ihdry		FLslider	"Dry Signal Level",					0,     1,    0,     5,        iddry,    500,     26,    0, 100
gkwet,ihwet		FLslider	"Wet Signal Level",					0,     1,    0,     5,        idwet,    500,     26,    0, 150
gkatt,ihatt		FLslider	"Attack Time",						.001, 10,   -1,     5,        idatt,    500,     26,    0, 200
gkrel,ihrel		FLslider	"Release Time",						.001, 10,   -1,     5,        idrel,    500,     26,    0, 250
gkamp,ihamp		FLslider	"Output Amplitude",					0,     2,    0,     5,        idamp,    500,     26,    0, 300

;SET_INITIAL_VALUES		VALUE | HANDLE
		FLsetVal_i   	.05, 	ihdlt
		FLsetVal_i   	0, 	ihfeedback
		FLsetVal_i   	1, 	ihdry
		FLsetVal_i   	1, 	ihwet
		FLsetVal_i   	.1, 	ihatt
		FLsetVal_i   	.1, 	ihrel
		FLsetVal_i   	1, 	ihamp

		FLpanel_end	;END OF PANEL CONTENTS

;INSTRUCTIONS AND INFO PANEL
				FLpanel	" ", 500, 280, 512, 0
;TEXT BOXES												TYPE | FONT | SIZE | WIDTH | HEIGHT | X | Y
ih		 	FLbox  	"          MIDI Keyboard Controlled Pitch Shifter             ", 	1,      5,     14,    490,    15,     5,  0
ih		 	FLbox  	"-------------------------------------------------------------", 	1,      5,     14,    490,    15,     5,  20
ih		 	FLbox  	"This example implements a pitch shifter effect based on the  ", 	1,      5,     14,    490,    15,     5,  40
ih		 	FLbox  	"algorithm introduced previously under 'Delay Effects'.       ", 	1,      5,     14,    490,    15,     5,  60
ih		 	FLbox  	"The innovation in this example is that the pitch shifter     ", 	1,      5,     14,    490,    15,     5,  80
ih		 	FLbox  	"effect is applied to a live input signal and that the        ", 	1,      5,     14,    490,    15,     5, 100
ih		 	FLbox  	"interval of pitch shift is determined by notes played on a   ", 	1,      5,     14,    490,    15,     5, 120
ih		 	FLbox  	"MIDI keyboard (middle C represents no pitch shift).          ", 	1,      5,     14,    490,    15,     5, 140
ih		 	FLbox  	"The number of pitch shifter voices is dynamic therefore      ", 	1,      5,     14,    490,    15,     5, 160
ih		 	FLbox  	"harmony chords are possible. The number of voices possible is", 	1,      5,     14,    490,    15,     5, 180
ih		 	FLbox  	"restricted only by available CPU.                            ", 	1,      5,     14,    490,    15,     5, 200
ih		 	FLbox  	"Attack and release controls are included to allow voices to  ", 	1,      5,     14,    490,    15,     5, 220
ih		 	FLbox  	"enter and leave according to a user defined amplitude        ", 	1,      5,     14,    490,    15,     5, 240
ih		 	FLbox  	"envelope.                                                    ", 	1,      5,     14,    490,    15,     5, 260

				FLpanel_end

				FLrun	;RUN THE FLTK WIDGET THREAD
;END OF FLTK INTERFACE CODE;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

		instr	2	;PLAYS FILE
gasig, gaignore	ins
		out	gasig * gkdry, gasig * gkdry
		endin

		instr	1				; PITCH SHIFTER
afeedback1	init	0	;SET INITIAL STATE FEEDBACK SIGNAL
afeedback2	init	0       ;SET INITIAL STATE FEEDBACK SIGNAL

ioct		octmidi
kratio		=	cpsoct(ioct)/cpsoct(8)	;RATIO OF NEW FREQ TO A DECLARED BASE FREQUENCY (MIDDLE C)
krate		=	(kratio-1)/gkdlt		;SUBTRACT 1/1 SPEED

aphase1		phasor	-krate			; MOVING PHASE 1-0
aphase2		phasor	-krate, .5		; MOVING PHASE 1-0 - PHASE OFFSET BY 180 DEGREES (.5 RADIANS)

agate1		tablei	aphase1, 1, 1, 0, 1	; WINDOW FUNC =TRIANGLE
agate2		tablei	aphase2, 1, 1, 0, 1	; WINDOW FUNC =TRIANGLE

aignore		delayr	.2			; ALLOC DELAY LINE
adelsig1	deltap3	aphase1 * gkdlt		; VARIABLE TAP
		delayw	gasig			; WRITE AUDIO TO THE BEGINNING OF THE DELAY BUFFER

aignore		delayr	.2				;ALLOC DELAY LINE
adelsig2	deltap3	aphase2 * gkdlt			;VARIABLE TAP
		delayw	gasig+ afeedback1+afeedback2	;WRITE AUDIO TO THE BEGINNING OF THE DELAY BUFFER

aGatedSig1	=	adelsig1 * agate1	;GATE 1ST SIGNAL
aGatedSig2	=	adelsig2 * agate2	;GATE 2ND SIGNAL
aGatedMix	=	(aGatedSig1 + aGatedSig2) * .5	;MIX AND ATTENUATE

afeedback1	=	aGatedSig1 * gkfeedback	;CREATE FEEDBACK SIGNAL
afeedback2	=	aGatedSig2 * gkfeedback	;CREATE FEEDBACK SIGNAL

iatt		init	i(gkatt)	;CONVERT ATTACK AND RELEASE CONTROLS TO I-RATE AND MAKE AVAILABLE AT I-TIME BY USING INIT
irel		init	i(gkrel)        ;CONVERT ATTACK AND RELEASE CONTROLS TO I-RATE AND MAKE AVAILABLE AT I-TIME BY USING INIT

kenv		expsegr	.001,iatt,1,irel,.001	;AMPLITUDE ENVELOPE IS CREATED
		outs	aGatedMix*kenv*gkwet*gkamp, aGatedMix*kenv*gkwet*gkamp
		endin

</CsInstruments>

<CsScore>
f 1 0 129 20 3       ; TRIANGLE WINDOW FUNCTION USED FOR AMPLITUDE ENVELOPING

;INSTR | START | DURATION
i  2       0       -1	;INSTRUMENT 2 PLAYS A HELD NOTE

f 0 3600	;'DUMMY' SCORE EVENT KEEPS REALTIME PERFORMANCE GOING FOR 1 HOUR
</CsScore>

</CsoundSynthesizer>